var toolbox = require("./toolbox.js")

var game = {   
    forza4 : [],
    nColonne : 7,
    nLinee : 6,
    player1 : "x", // "x"
    player2 : "o", // "o"    

    init: function()
    {
        this.forza4 = toolbox.initTable(this.nLinee, this.nColonne, 0);
    },
    
    /**
     * Permette di mostrare una tabella in stile forza 4
     */
    mostraForza4: function()
    {
        for(var i = 0; i < this.forza4.length; i++)
        {
            var linea = "";
            for(var j = 0; j < this.forza4[i].length; j++)
            {
                linea += "| ";
                if(this.forza4[i][j] === 0) 
                {
                    linea += "_";
                }
                else if(this.forza4[i][j] === 1) 
                {
                    linea += this.player1;
                }
                else if(this.forza4[i][j] === 2) 
                {
                    linea += this.player2;
                }
                linea += "| ";
            }
            console.log(linea);
        }
    },

    /**
     * Funzione che permette di ottenere la prima linea VUOTA di una colonna
     * @param {*} colonna ritorna -1 se la colonna è piena
     * @returns 
     */
    ritornaLineaCasellaVuotaColonna: function(colonne)
    {
        for(var i = this.nLinee-1; i >= 0; i--)
        {
            if(this.verificaCasellaVuota(i, colonne)) return i;
        }
        return -1;
    },

    /**
     * Funzione che permette di otenere se una casella è vuota (ritorna true/false)
     * @param {*} linea 
     * @param {*} colonna 
     * @returns 
     */
    verificaCasellaVuota: function(linea, colonna)
    {
        return this.forza4[linea][colonna-1] === 0;
    },

    verificaFineDelGioco: function(player)
    {
        if(this.verificaLineaFineGioco(player) ||
            this.verificaColonnaFineGioco(player) || 
            this.verificaDiagonaleFineGioco(player)) 
        {
            return true;
        }
        return false;
    },

    /**
     * Funzione che permette di verificare se un giocatore ha vinto su una linea
     * @param {*} player 
     * @returns 
     */
    verificaLineaFineGioco: function(player)
    {
        for(var i = this.nLinee-1; i >= 0; i--)
        {
            for(var i=this.nLinee-1;i>=0; i--)
            {
                for (var j=0;j<this.nColonne-3;j++)
                {
                    if( this.forza4[i][j] === player &&
                        this.forza4[i][j+1] === player &&
                        this.forza4[i][j+2] === player &&
                        this.forza4[i][j+3] === player
                        )
                    {
                        return true;
                    }
                }
            }
        }
        return false;
    },

    /**
     * Funzione che permette di verificare se un giocatore ha vinto su una colonna
     * @param {*} player 
     * @returns 
     */
    verificaColonnaFineGioco: function(player)
    {
        for(var i = 0; i<this.nColonne;i++)
        {
            for(var j=this.nLinee-4;j>=0;j--)
            {
                if( this.forza4[j][i] === player &&
                    this.forza4[j+1][i] === player &&
                    this.forza4[j+2][i] === player &&
                    this.forza4[j+3][i] === player
                    ) return true;
            }
        }
    },

    /**
     * Funzione che permette di verificare se un giocatore ha vinto sulla diagonale
     * @param {*} player 
     * @returns 
     */
    verificaDiagonaleFineGioco: function(player)
    {
        for(var i=this.nLinee-1;i>=3; i--)
        {
            for (var j=0;j<this.nColonne;j++)
            {
                if( this.forza4[i][j] === player &&
                    this.forza4[i-1][j+1] === player &&
                    this.forza4[i-2][j+2] === player &&
                    this.forza4[i-3][j+3] === player
                    ) return true;
                if( this.forza4[i][j] === player &&
                    this.forza4[i-1][j-1] === player &&
                    this.forza4[i-2][j-2] === player &&
                    this.forza4[i-3][j-3] === player
                    ) return true;
            }
        }
        return false;
    },
    
    scegliereColonna: function()
    {
        return parseInt(toolbox.scriviStringa("Quale colonna scegli?"));
    },

    giocaColpo : function(player, lineaVuota, colonna)
    {
        this.forza4[lineaVuota][colonna-1] = player;
    }
}

module.exports = game;